@@ -13,7 +13,7 @@ require 'amstd/rbparams'
 
 module RubyEmuration
 
-  WITH_EXT = /\.rb\z|\.#{RubyParams::DL_EXT}\z/o
+  WITH_EXT = /\.rb\z|\.#{RubyParams::DLEXT}\z/o
 
   def find_feature( feature )
     with_ext( feature ) do |fn|
