@@ -4,6 +4,8 @@
 #   Copyright (c) 1999 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
 #
 
+require 'amstd/symbol'
+
 
 class Object
 
@@ -12,45 +14,45 @@ class Object
   def _name2str( arg )
     case arg
     when String then arg
-    when Fixnum then arg.id2name
+    when Symbol then arg.id2name
     else
       raise TypeError,
-        "wrong arg type #{arg.type} for expected String/Fixnum"
+        "wrong arg type #{arg.type} for expected String/Symbol"
     end
   end
 
   def _type2str( arg )
     case arg
     when String then arg
-    when Fixnum then arg.id2name
+    when Symbol then arg.id2name
     when Module then arg.name
     when Array
       arg.collect{|i| _type2str i }.join(',')
     else
       raise TypeError,
-        "wrong arg type #{arg.type} for expected String/Fixnum/Module/Array"
+        "wrong arg type #{arg.type} for expected String/Symbol/Module/Array"
     end
   end
 
 
   def _name2simbol( arg )
     case arg
-    when Fixnum then arg
+    when Symbol then arg
     when String then arg.intern
     else
       raise TypeError,
-        "wrong arg type #{arg.type} for expected String/Fixnum"
+        "wrong arg type #{arg.type} for expected String/Symbol"
     end
   end
 
   def _type2simbol( arg )
     case arg
     when String then arg.intern
-    when Fixnum then arg
+    when Symbol then arg
     when Module then arg.name.intern
     else
       raise TypeError,
-        "wrong arg type #{arg.type} for expected String/Fixnum/Module"
+        "wrong arg type #{arg.type} for expected String/Symbol/Module"
     end
   end
 
