@@ -129,7 +129,11 @@ do_reduce(v, ruleno, debug)
     reduce_to  = RARRAY(v->reduce_table)->ptr[ruleno+1];
     method_id  = RARRAY(v->reduce_table)->ptr[ruleno+2];
     len = FIX2LONG(reduce_len);
+#ifdef SYM2ID
+    mid = SYM2ID(method_id);
+#else
     mid = (ID)FIX2LONG(method_id);
+#endif
 
     if (len == 0) {
         tmp = Qnil;
