@@ -1,4 +1,5 @@
 
+$LOAD_PATH.unshift '.'
 require 'amstd/inst'
 
 class TMAILinstaller < Installer
@@ -6,17 +7,17 @@ class TMAILinstaller < Installer
   def com_setup
     setup_library 'tmail'
     setup_library 'racc'
-    chdir( 'strscan' ) do
+    into_dir( 'strscan' ) do
       extconf
       make
     end
   end
 
   def com_install
-    %w( amstd racc tmail ).each do |lib|
+    %w( amstd tmail racc ).each do |lib|
       install_library lib
     end
-    chdir( 'strscan' ) do
+    into_dir( 'strscan' ) do
       make_install
     end
   end
