@@ -8,10 +8,9 @@
 #   the GNU Lesser General Public License version 2 or later.
 #
 
+require 'sync'
 require 'delegate'
-
 require 'amstd/to_s'
-
 require 'tmail/mailp'
 require 'tmail/encode'
 
@@ -71,6 +70,8 @@ module TMail
   CONTROL  = "\\\x00-\\\x20\\\x7f-\\\xff"
   INSECURE = /[#{Regexp.quote TSPECIAL}#{CONTROL}]/o
 
+  ENCODED = /=\?[^\s?=]+\?[qb]\?[^\s?=]+\?=/i
+
   class << self
 
   def msgid?( str )
@@ -109,6 +110,10 @@ module TMail
     end
   end
 
+  def encoded?( str )
+    ENCODED === str
+  end
+
 
   # internal use
 
@@ -146,15 +151,17 @@ class HeaderField
     @body = fbody
     @strict = strict
 
+    @illegal = false
     @parsed = false
     @parsing = false
-    @written = false
+    # @sync = Sync.new
   end
 
 
   R     = (1 << 0)  # readable
   W     = (1 << 1)  # writable
   NODUP = (1 << 2)  # dup?
+  NOEQL = (1 << 3)  # used by eql? ?
 
   class << self
 
@@ -176,9 +183,11 @@ class HeaderField
 
 
     def parse_on_rw
+      @attrs = nil
     end
 
     def parse_on_create
+      @attrs = nil
     end
 
     def header_attr( name, type, flags, ali = nil )
@@ -187,13 +196,20 @@ class HeaderField
       r = (flags & R != 0)
       w = (flags & W != 0)
       c = (flags & NODUP != 0)
+      a = (flags & NOEQL != 0)
+
+      unless a then
+        if @attrs then
+          @attrs.push name
+        else
+          @attrs = [name]
+        end
+      end
 
       if r then
         module_eval %-
           def #{name}
-            unless @parsed or @parsing then
-              parse
-            end
+            ensure_parsed
             @#{name}
           end
         -
@@ -202,9 +218,7 @@ class HeaderField
       if w then
         module_eval %-
           def #{name}=( arg )
-            unless @parsed or @parsing then
-              parse
-            end
+            ensure_parsed
             @#{name} = arg
           end
         -
@@ -218,56 +232,113 @@ class HeaderField
         end
       end
     end
+
+    def redefine_eql
+      bug! unless @attrs
+
+      eqls = @attrs.collect {|at| sprintf( '%s.eql?(other.%s)', at, at ) }
+      module_eval %-
+        def eql?( other )
+          super and
+          #{eqls.join(' and ')}
+        end
+      -
+
+      eqls = @attrs.collect {|at| sprintf( '%s == other.%s', at, at ) }
+      module_eval %-
+        def ==( other )
+          super and #{eqls.join(' and ')}
+        end
+      -
+    end
+
+    def empty_able
+      bug! unless @attrs
+
+      module_eval %-
+        def empty?
+          ensure_parsed
+          if @illegal then
+            /\A\s*\z/ === @body
+          else
+            not ( #{@attrs.join(' or ')} )
+          end
+        end
+      -
+    end
         
   end
 
-  # abstract parse
-
 
   def hash
     @name.hash
   end
 
   def eql?( oth )
-    self.type === oth and @name == oth.name
+    @name.downcase == oth.name.downcase
   end
 
-  def ==( oth )   # don't alias
-    eql? oth
+  alias == eql?
+
+
+  def illegal?
+    @illegal
   end
-    
+
+  def empty?
+    ensure_parsed
+    @illegal and (/\A\s*\z/ === @body)
+  end
+
 
   def name
     @name.dup
   end
 
-
   def body
-    ret = ''
-    v = HFdecoder.new( ret )
-    unless @parsed or @parsing then
-      parse
-    end
+    ensure_parsed
+    v = HFdecoder.new( '' )
     do_accept v
     v.write_in
-    ret
   end
 
+  def body=( str )
+    @body = str
+    @parsed = false
+    @illegal = false
+  end
 
   def accept( strategy )
+    ensure_parsed
     strategy.header_name @name
-    unless @parsed or @parsing then
-      parse
-    end
     do_accept strategy
   end
 
   ::TMail.define_header_to_s self, true
 
-  def do_accept( strategy )
-    strategy.header_body body
+
+  private
+
+  def ensure_parsed
+    begin
+      Thread.critical = true   #@sync.lock :EX
+      unless @parsed or @parsing then
+        @parsing = true
+        Thread.critical = false
+        parse
+        Thread.critical = true
+        @parsing = false
+        @parsed = true
+      end
+    ensure
+      Thread.critical = false   # @sync.unlock
+    end
   end
 
+  # abstract parse
+
+  # abstract do_accept
+
 end
 
 
@@ -276,16 +347,10 @@ end
 
 module NoAttrHeader
 
-  def NoAttrHeader.append_features( mod )
-  mod.module_eval <<'_'
-
-
-  header_attr :body, String, R | W
-
-  def eql?( other )
-    super and
-    body == other.body
-  end
+      def append_features( mod ) super; mod.
+  
+  header_attr :body, String, R | W    ; mod.
+  redefine_eql                          end
 
   private
 
@@ -294,14 +359,8 @@ module NoAttrHeader
   end
 
   def parse
-    @parsing = true
-    init
+    init_real
     TMail::HFdecoder.decode( @body )
-    @parsing = false
-    @parsed = true
-  end
-
-_
   end
 
 end
@@ -313,7 +372,9 @@ class StringH < HeaderField
 
   include NoAttrHeader
 
-  def init
+  private
+
+  def init_real
   end
 
 end
@@ -325,29 +386,34 @@ class StructH < HeaderField
 
   parse_on_rw
 
-  header_attr :comments, Array, R | NODUP
+  header_attr :comments, Array, R | NODUP | NOEQL
 
 
   private
 
-  def init
+  def init_real
     @comments = []
+    init
   end
 
   def parse
-    @parsing = true
-    init
-    if @strict then
+    pre = nil
+
+    begin
+      init_real
       Mailp.parse( @body, self, nil )
-    else
-      begin
-        Mailp.parse( @body, self, nil )
-      rescue ParseError, ScanError
-        ;
+    rescue ParseError, ScanError
+      if not pre and ::TMail.encoded?( @body ) then
+        pre = @body
+        @body = HFdecoder.decode( pre )
+        retry
+      elsif pre then
+        @body = pre
       end
+
+      @illegal = true
+      raise if @strict
     end
-    @parsing = false
-    @parsed = true
   end
 
 end
@@ -359,6 +425,11 @@ class UnknownH < StructH
 
   include NoAttrHeader
 
+  private
+
+  def init
+  end
+
 end
 
 
@@ -372,12 +443,16 @@ class DateH < StructH
 
   def date=( arg )
     @date = arg.localtime
-    @written = true
   end
 
-  def eql?( oth )
-    super and
-    @date == oth.date
+  redefine_eql
+  empty_able
+
+
+  private
+
+  def init
+    @date = nil
   end
 
   def do_accept( strategy )
@@ -396,17 +471,13 @@ class RetpathH < StructH
   header_attr :route, Array,  R | NODUP, [:routes]
   header_attr :addr,  String, R | W
 
-  def eql?( oth )
-    super and
-    self.addr == oth.addr and
-    self.routes == oth.routes
-  end
+  redefine_eql
+  empty_able
 
 
   private
 
   def init
-    super
     @route = []
     @addr = nil
   end
@@ -452,6 +523,10 @@ class Address
   attr_writer :route   # internal use only
 
 
+  def Address.join( arr )
+    arr.collect {|i| ::TMail.quote i }.join('.')
+  end
+
   def local
     @local.collect {|i| ::TMail.quote i }.join('.')
   end
@@ -578,9 +653,14 @@ class SaddrH < StructH
 
   header_attr :addr, Address, R | W
 
-  def eql?( oth )
-    super and
-    self.addr == oth.addr
+  redefine_eql
+  empty_able
+
+
+  private
+
+  def init
+    @addr = nil
   end
 
   def do_accept( stra )
@@ -609,16 +689,13 @@ class MaddrH < StructH
 
   header_attr :addrs, Array, R | NODUP
 
-  def eql?( oth )
-    super and
-    addrs == oth.addrs
-  end
+  redefine_eql
+  empty_able
 
 
   private
 
   def init
-    super
     @addrs = []
   end
 
@@ -665,10 +742,9 @@ class RefH < StructH
 
   header_attr :refs, Array, R | NODUP
 
-  def eql?( oth )
-    super and
-    self.refs == oth.refs
-  end
+  redefine_eql
+  empty_able
+
 
   def each_msgid
     refs.each do |i|
@@ -686,7 +762,6 @@ class RefH < StructH
   private
 
   def init
-    super
     @refs = []
   end
 
@@ -717,16 +792,13 @@ class KeyH < StructH
 
   header_attr :keys, Array, R | NODUP
 
-  def eql?( oth )
-    super and
-    self.keys == oth.keys
-  end
+  redefine_eql
+  empty_able
 
 
   private
 
   def init
-    super
     @keys = []
   end
 
@@ -759,23 +831,13 @@ class RecvH < StructH
   header_attr :for_,  String, R | W,        [:ford, :for_domain]
   header_attr :date,  Time,   R | W | NODUP
 
-
-  def eql?( oth )
-    super and
-    from  == oth.from and
-    by    == oth.by and
-    via   == oth.via and
-    with  == oth.with and
-    msgid == oth.msgid and
-    for_  == oth.for_ and
-    date  == oth.date
-  end
+  redefine_eql
+  empty_able
 
 
   private
 
   def init
-    super
     @from = @by = @via = @with = @msgid = @for_ = nil
     @with = []
   end
@@ -823,9 +885,14 @@ class MsgidH < StructH
 
   header_attr :msgid, String, R | W
   
-  def eql?( oth )
-    super and
-    msgid == oth.msgid
+  redefine_eql
+  empty_able
+
+
+  private
+
+  def init
+    @msgid = nil
   end
 
   def do_accept( stra )
@@ -844,14 +911,14 @@ class EncH < StructH
   header_attr :encrypter, String, R | W
   header_attr :keyword,   String, R | W
 
-  def eql?( oth )
-    super and
-    self.encrypter == oth.encrypter and
-    self.keyword   == oth.keyword
-  end
+  redefine_eql
+  empty_able
+
+
+  private
 
   def init
-    super
+    @encrypter = nil
     @keyword = nil
   end
 
@@ -877,14 +944,19 @@ class VersionH < StructH
   header_attr :major, :Integer, R | W | NODUP
   header_attr :minor, :Integer, R | W | NODUP
 
+  redefine_eql
+  empty_able
+
   def version
     sprintf( '%d.%d', major, minor )
   end
 
-  def eql?( oth )
-    super and
-    self.major == oth.major and
-    self.minor == oth.minor
+  
+  private
+
+  def init
+    @major = nil
+    @minor = nil
   end
 
   def do_accept( stra )
@@ -905,22 +977,17 @@ class CTypeH < StructH
   header_attr :sub,    :String, R | W
   header_attr :params, :Hash,   R | NODUP
 
+  redefine_eql
+  empty_able
+
   def []( key )
     params[key]
   end
 
-  def eql?( oth )
-    super and
-    main == oth.main and
-    sub  == oth.sub  and
-    params == oth.params
-  end
-
 
   private
 
   def init
-    super
     @main = @sub = nil
     @params = {}
   end
@@ -947,9 +1014,14 @@ class CEncodingH < StructH
 
   header_attr :encoding, :String, R | W
 
-  def eql?( oth )
-    super and
-    encoding == oth.encoding
+  redefine_eql
+  empty_able
+
+
+  private
+
+  def init
+    @encoding = nil
   end
 
   def do_accept( stra )
@@ -972,17 +1044,13 @@ class CDispositionH < StructH
     params[key]
   end
 
-  def eql?( oth )
-    super and
-    self.disposition == oth.disposition and
-    self.params == oth.params
-  end
+  redefine_eql
+  empty_able
   
 
   private
 
   def init
-    super
     @disposition = nil
     @params = {}
   end
